@@ -74,12 +74,12 @@
 */
 
 
+#include "local.h"
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
 #include <ctype.h>
 #include <limits.h>
-#include "local.h"
 
 /**
  * \brief Gets the boolean value from the given ASCII string.
@@ -154,10 +154,10 @@ int snd_config_get_card(const snd_config_t *conf)
 	long v;
 	int err;
 
-	if ((err = snd_config_get_integer(conf, &v)) < 0) {
-		if ((err = snd_config_get_string(conf, &str)) < 0) {
-			snd_config_get_id(conf, &id);
-			SNDERR("Invalid field %s", id);
+	if (snd_config_get_integer(conf, &v) < 0) {
+		if (snd_config_get_string(conf, &str)) {
+			if (snd_config_get_id(conf, &id) >= 0)
+				SNDERR("Invalid field %s", id);
 			return -EINVAL;
 		}
 		err = snd_card_get_index(str);
@@ -645,7 +645,7 @@ static int string_from_integer(char **dst, long v)
 }
 #endif
 
-int _snd_func_private_data(snd_config_t **dst, snd_config_t *src,
+static int _snd_func_private_data(snd_config_t **dst, snd_config_t *src,
 			   snd_config_t **private_data, const char *id)
 {
 	int err;
@@ -1031,6 +1031,14 @@ int snd_func_card_name(snd_config_t **dst, snd_config_t *root,
 SND_DLSYM_BUILD_VERSION(snd_func_card_name, SND_CONFIG_DLSYM_VERSION_EVALUATE);
 #endif
 
+#ifdef DOXYGEN
+/* For consistency with the PCM Interface module, include documentation even
+ * when PCM module is not included in the build. */ 
+#ifndef BUILD_PCM
+#define BUILD_PCM
+#endif
+#endif /* DOXYGEN */
+
 #ifdef BUILD_PCM
 
 /**
